/* Copyright © 2013 Matthew Champion */
/*  */

package com.mattunderscore.http.headers;

import java.util.ArrayList;
import java.util.Collection;

/**
 * Parse an accept encoding field header string to a structured representation.
 * 
 * A special note is required for the identity encoding. The identity encoding is always acceptable
 * unless it explicitly or by wild card has a qualifier of 0. Where the identity is acceptable it
 * shall have the quality value a tenth less than the smallest other quality value. The quality
 * value is not part of the specification.
 * 
 * @author Matt Champion
 * @since 0.1.2
 */
public class EncodingParser implements HeaderParser<QEncoding>
{

    public EncodingParser()
    {
    }

    @Override
    public boolean isCorrectHeader(String header)
    {
        return "Accept-Encoding".equalsIgnoreCase(header);
    }

    @Override
    public Collection<QEncoding> parseAll(String header) throws UnParsableHeaderException
    {
        Collection<QEncoding> qEcodings = new ArrayList<QEncoding>();
        String[] types = header.split(",");
        boolean addIdentity = true;
        double smallestQualifier = 10.0;
        for (int i = 0; i < types.length; i++)
        {
            QEncoding encoding = parse(types[i]);
            if (encoding != null)
            {
                if (encoding.sameEncoding(new QEncoding("identity", 1.0)))
                {
                    addIdentity = false;
                }
                else if (encoding.isAnyEncoding())
                {
                    addIdentity = false;
                }
                else if (addIdentity && encoding.getQualifier() < smallestQualifier)
                {
                    smallestQualifier = encoding.getQualifier();
                }
                qEcodings.add(encoding);
            }
        }
        if (addIdentity)
        {
            qEcodings.add(new QEncoding("identity", smallestQualifier / 10.0));
        }
        return qEcodings;
    }

    @Override
    public QEncoding parse(String header) throws UnParsableHeaderException
    {
        if (header == null)
        {
            throw new UnParsableHeaderException("Null Pointer.");
        }
        else if ("".equals(header.trim()))
        {
            return null;
        }
        else
        {
            String[] headers = header.split(";q=");
            if (headers.length == 2)
            {
                double qualifier = Double.parseDouble(headers[1]);
                if (qualifier < 0.0 || qualifier > 1.0)
                {
                    throw new UnParsableHeaderException("Invalid qualifier range.");
                }
                else
                {
                    return new QEncoding(headers[0], qualifier);
                }
            }
            else
            {
                return new QEncoding(headers[0], 1.0);
            }
        }
    }
}
